/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Preferences Toolbar 2.
 *
 * The Initial Developer of the Original Code is
 * Aaron Andersen.
 * Portions created by the Initial Developer are Copyright (C) 2___
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): Aaron Andersen <aaron@xulplanet.com>
 *                 Manuel Reimer <Manuel.Reimer@gmx.de>
 *                 Stephen Clavering <mozilla@clav.me.uk>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

function prefbarClearCookies() {
    var cookieman = Components.classes["@mozilla.org/cookiemanager;1"]
                            .getService(Components.interfaces.nsICookieManager);
    cookieman.removeAll();
}


function prefbarClearHistory() {
    // builds from 20040210ish onward use the second class.  the relevant parts
    // of the nsIBrowserHistory are not altered though.
    var class1 = "@mozilla.org/browser/global-history;1";
    var class2 = "@mozilla.org/browser/global-history;2";
    var history = (class2 in Components.classes) ? class2 : class1;
    history = Components.classes[history].getService(Components.interfaces.nsIBrowserHistory);
    history.removeAllPages();

    //for firefox.  used to clear the history on back/forward buttons, and the <browser/> session history
    var os = Components.classes["@mozilla.org/observer-service;1"]
	.getService(Components.interfaces.nsIObserverService);
    os.notifyObservers(null, "browser:purge-session-history", "");
}


function prefbarClearLocationBar() {
    // Mozilla up to 1.7 uses the class "urlbarhistory"
    var classID = Components.classes['@mozilla.org/browser/urlbarhistory;1'];
    if (classID) {
	var urlbarHistory = classID.getService(Components.interfaces.nsIUrlbarHistory);
	urlbarHistory.clearHistory();
    }
    // Mozilla since version 1.8 clears the history via RDF functions
    else {
	var RDF = Components.classes["@mozilla.org/rdf/rdf-service;1"]
	    .getService(Components.interfaces.nsIRDFService);
	var localStore = RDF.GetDataSource("rdf:local-store");
	var RDFC = Components.classes["@mozilla.org/rdf/container-utils;1"]
	    .getService(Components.interfaces.nsIRDFContainerUtils);
	var urlBarHist = RDFC.MakeSeq(localStore, RDF.GetResource("nc:urlbar-history"));
	for (var i = urlBarHist.GetCount(); i > 0; --i)
	    urlBarHist.RemoveElementAt(i, true);
    }
    navigator.preference('general.open_location.last_url', '');
}


function prefbarClearCache(aType) {
    var classID = Components.classes["@mozilla.org/network/cache-service;1"];
    var cacheService = classID.getService(Components.interfaces.nsICacheService);
    cacheService.evictEntries(aType);
}

function prefbarClearMemCache() {
    prefbarClearCache(Components.interfaces.nsICache.STORE_IN_MEMORY);
}

function prefbarClearDiskCache() {
    prefbarClearCache(Components.interfaces.nsICache.STORE_ON_DISK);
}

function prefbarClearAllCache() {
    prefbarClearMemCache();
    prefbarClearDiskCache();
}

function prefbarKillFlash() {
    var frames;
    frames = prefbarGetFrames(window._content.window);
    for (var frameindex in frames) {
        var e_top;
        var e_next;
        var page = frames[frameindex].document;
        var flashes = page.getElementsByTagName("embed");

        for(var i=0; i<flashes.length; i++) {
            var current = flashes[0];

            if(current.getAttribute("type") =="application/x-shockwave-flash") {
                var height = current.getAttribute("height");
                var width = current.getAttribute("width");

                if(current.parentNode.nodeName.toLowerCase() == "object") {
                    e_top = current.parentNode.parentNode;
                    e_next = current.parentNode;
                }
                else {
                    e_top = current.parentNode;
                    e_next = current;
                }

                if(height && width) {
                    var div = page.createElement("DIV");
                    var text = page.createTextNode(" ");
                    div.appendChild(text);

                    e_top.replaceChild(div, e_next);

                    div.setAttribute("style", "height: " + height + "px; width: " + width + "px; border: 1px solid black;");
                }
                else {
                    e_top.removeChild(current);
                }

                i--;
            }
        }
    }
}

function prefbarGetFrames(owindow) {
    var winarray = new Array();
    winarray.push(owindow);
    for (var index = 0; index < owindow.frames.length; index++)
        winarray = winarray.concat(getFrames(owindow.frames[index]));
    return winarray;
}


//
// Proxy serverlist
//

function prefbarSetProxy(str) {
    if (str == "") {
	navigator.preference("network.proxy.type", 0);
	/* Do we need to clear preference settings? */
    }
    else {
	var args = str.split(":");

	if (args.length != 2) {
	    alert('Wrong proxy setting: "' + str + '"');
	    return;
	}

	// Maybe we shouldn't set the Proxy Type to "Manual" automatically, since
	// some users may want to use "Proxy Serverlist" *and* "Proxy Menulist" at once...
	navigator.preference("network.proxy.type", 1);

	// Set HTTP-Proxy
	navigator.preference("network.proxy.http", args[0]);
	navigator.preference("network.proxy.http_port", Number(args[1]));
	// Set SSL-Proxy
	navigator.preference("network.proxy.ssl", args[0]);
	navigator.preference("network.proxy.ssl_port", Number(args[1]));
	// Set FTP-Proxy
	navigator.preference("network.proxy.ftp", args[0]);
	navigator.preference("network.proxy.ftp_port", Number(args[1]));
	// Set Gopher-Proxy
	navigator.preference("network.proxy.gopher", args[0]);
	navigator.preference("network.proxy.gopher_port", Number(args[1]));
    }
}

function prefbarGetProxy() {
    if (navigator.preference("network.proxy.type") != 1) {
	return "";
    }

    return navigator.preference("network.proxy.http") + ":" +
	navigator.preference("network.proxy.http_port");
}

//
// "Pipelining" checkbox
//

function prefbarSetPipelining(value) {
    navigator.preference("network.http.proxy.pipelining", value);
    navigator.preference("network.http.pipelining", value);
}

function prefbarGetPipelining() {
    return (navigator.preference("network.http.proxy.pipelining") && navigator.preference("network.http.pipelining"));
}

//
// "Resize" menulist
//

function prefbarSetResolution(value) {
    var valueArray = value.split("x");
    window.outerWidth = valueArray[0];
    window.outerHeight = valueArray[1];
}

function prefbarGetResolution() {
    return window.outerWidth + "x" + window.outerHeight;
}

//
// "Save Page" button
//

var prefbarSPFilename = null;
var prefbarSPOldGetDefaultFileName = null;
var prefbarSPGetDefaultFileName = function(param1, param2, param3, param4) {
    if (prefbarSPFilename) {
	var retval = prefbarSPFilename;
	prefbarSPFilename = null;
	return retval;
    }
    else
        return prefbarSPOldGetDefaultFileName(param1, param2, param3, param4);
}

function prefbarSavePage() {
    var title = window._content.document.title;
    var uri = window._content.location;
    var doc = window._content.document;
    var rchar = "_";

    title = title.replace(/\/|\\|:|\*|\?|\"|<|>|\|/g, rchar);
    title = title.replace(/\s/g, rchar);
    if (title.match(new RegExp("^" + rchar + "+$"))) title = "";

    // Filename is the modified title
    prefbarSPFilename = title;

    if (!prefbarSPOldGetDefaultFileName) {
	if ("getDefaultFileName" in this) {
	    prefbarSPOldGetDefaultFileName = this.getDefaultFileName;
	    this.getDefaultFileName = prefbarSPGetDefaultFileName;
	}
	else {
	    alert("PrefBar: Save Page feature not available for the mozilla version you use!");
	    return;
	}
    }

    // Save function used by Mozilla up to 1.7
    if ("saveInternal" in this)
	saveInternal(uri, doc, null, false);
    // Save function used by Mozilla since 1.8 and SeaMonkey
    else if ("internalSave" in this)
	internalSave(uri, doc, null, false, null, null);
    else
	alert("PrefBar: Save Page feature not available for the mozilla version you use!");
}



//
// external application support
//

function prefbarExecute(path, arguments) {
    if (arguments == undefined) arguments = new Array;

    var executable = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
    executable.initWithPath(path);

    if (!executable.exists()) {
	alert("PrefBar: Error - executable '" + path + "' not found");
	return;
    }
    try {
	var process = Components.classes["@mozilla.org/process/util;1"].createInstance(Components.interfaces.nsIProcess);

	process.init(executable);

	var pid = new Object();
	process.run(false, arguments, arguments.length, pid);
    }
    catch (ex) {
	alert("PrefBar: Error - couldn't execute '" + path + "'");
	return;
    }
}

//
// Flash Checkbox
//

function prefbarSetFlash(value)
{
    var Plugin = prefbarGetPluginPath();
    var PluginName = prefbarGetPluginName();

    if (!Plugin) {
        alert("No Flash Plugin found!");
	return;
    }

    var strPluginPath = Plugin.path;
        
    var destName;
    if (value == true) {
	Plugin.append(PluginName + ".prefbar");
	destName = PluginName;
    }
    else {
	Plugin.append(PluginName);
	destName = PluginName + ".prefbar";
    }

    if (!Plugin.exists()) {
	alert("No Flash Plugin found!");
	return;
    }

    try {
        Plugin.moveTo(null, destName);
    }
    catch (e) { 
        alert("Failed to disable/enable flash plugin. You seem to have no write access to the directory '" + strPluginPath + "'");
    }
}

function prefbarGetFlash()
{
    var Plugin = prefbarGetPluginPath();
    if (!Plugin) return false;

    Plugin.append(prefbarGetPluginName());
    if (Plugin.exists())
	return true;
    else
	return false;
}

function prefbarGetPluginName()
{
	if (window.navigator.platform.toLowerCase().indexOf("win") != -1)
		return "npswf32.dll";
	else
		return "libflashplayer.so";
}

// We return the plugin path (as nsILocalFile) which either contains a enabled
// or a disabled flash plugin.
function prefbarGetPluginPath() {
    var dirService = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties);
    var PluginsDL = dirService.get("APluginsDL", Components.interfaces.nsISimpleEnumerator);

    var PluginName = prefbarGetPluginName();
    var PluginNames = [PluginName, PluginName + ".prefbar"];

    while (PluginsDL.hasMoreElements()) {
        var PluginDir = PluginsDL.getNext().QueryInterface(Components.interfaces.nsILocalFile);

        for (var PluginIndex in PluginNames) {
            var TmpTestDir = PluginDir.clone();
            TmpTestDir.append(PluginNames[PluginIndex]);
            if (TmpTestDir.exists()) {
                return PluginDir;
            }
        }
    }
    return false;
}
